//+------------------------------------------------------------------+
//|                                           Schaff Trend Cycle.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  LimeGreen
#property indicator_color2  Orange
#property indicator_color3  Orange
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2
#property indicator_minimum  -5
#property indicator_maximum 105

//
//
//
//
//

extern string TimeFrame = "Current time frame";
extern int STCPeriod    = 10;
extern int FastMAPeriod = 12;
extern int SlowMAPeriod = 26;
extern int STCPrice     = PRICE_CLOSE;
extern int STCDepth     = 1;

//
//
//
//
//

double stcBuffer[];
double stcBufferDA[];
double stcBufferDB[];
double slope[];
string indicatorFileName;
int    timeFrame;
bool   returnBars;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,stcBuffer);
   SetIndexBuffer(1,stcBufferDA);
   SetIndexBuffer(2,stcBufferDB);
   SetIndexBuffer(3,slope);
      STCDepth = MathMax(MathMin(STCDepth,10),1);
         indicatorFileName = WindowExpertName();
         timeFrame         = stringToTimeFrame(TimeFrame);
         returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
   IndicatorShortName("Schaff Trend Cycle depth "+STCDepth+" ("+STCPeriod+","+FastMAPeriod+","+SlowMAPeriod+")");
   return(0);
}

int deinit()
{
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars > 0) counted_bars--;
           int limit = MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { stcBuffer[0] = MathMin(limit+1,Bars-1); return(0); }
           if (timeFrame!=Period())
           {
               limit = MathMax(MathMin(Bars-IndicatorCounted(),Bars-1),MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
               if (slope[limit] ==-1) CleanPoint(limit,stcBufferDA,stcBufferDB);
               for(int i = limit; i >= 0; i--)
               {
                  int y = iBarShift(NULL,timeFrame,Time[i]);
                     stcBuffer[i]   = iCustom(NULL,timeFrame,indicatorFileName,"",STCPeriod,FastMAPeriod,SlowMAPeriod,STCPrice,STCDepth,0,y);
                     slope[i]       = iCustom(NULL,timeFrame,indicatorFileName,"",STCPeriod,FastMAPeriod,SlowMAPeriod,STCPrice,STCDepth,3,y);
                     stcBufferDA[i] = EMPTY_VALUE;
                     stcBufferDB[i] = EMPTY_VALUE;
                        if (slope[i]==-1) PlotPoint(i,stcBufferDA,stcBufferDB,stcBuffer);
               }
               return(0);
            }  

   //
   //
   //
   //
   //
   
      if (slope[limit] ==-1) CleanPoint(limit,stcBufferDA,stcBufferDB);
      for(i = limit; i >= 0; i--)
      {
         double tprice = iMA(NULL,0,1,0,MODE_SMA,STCPrice,i);
         for (int k=0; k<STCDepth; k++)
         {
            double macd   = iEma(tprice,FastMAPeriod,i,k*2+0)-iEma(tprice,SlowMAPeriod,i,k*2+1);
                   tprice = iSchaff(macd,STCPeriod,i,k);
         }                   
         stcBuffer[i]   = tprice;
         stcBufferDA[i] = EMPTY_VALUE;
         stcBufferDB[i] = EMPTY_VALUE;
         slope[i]       = slope[i+1];
         if (stcBuffer[i]>stcBuffer[i+1]) slope[i] =  1;
         if (stcBuffer[i]<stcBuffer[i+1]) slope[i] = -1;
            if (slope[i]==-1) PlotPoint(i,stcBufferDA,stcBufferDB,stcBuffer);
      }            
      return(0);
}


//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double aSchaff[][50];
#define sPrice  0
#define stoch11 1
#define stoch12 2
#define stoch21 3
#define stoch22 4

double iSchaff(double price, int stcPeriod, int i, int instance)
{
   if (ArrayRange(aSchaff,0)!=Bars) ArrayResize(aSchaff,Bars); i = Bars-i-1; instance *= 5;
      aSchaff[i][instance+0] = price;
      
         //
         //
         //
         //
         //
         
         double minval = aSchaff[i][instance+sPrice];
         double maxval = aSchaff[i][instance+sPrice];
            for (int k=1;k<stcPeriod && (i-k)>=0; k++)
            {
               minval = MathMin(aSchaff[i-k][instance+sPrice],minval);
               maxval = MathMax(aSchaff[i-k][instance+sPrice],maxval);
            }
            maxval -= minval;
            if (maxval > 0)
                  aSchaff[i][instance+stoch11] = 100*((aSchaff[i][instance+sPrice]-minval)/maxval);
            else  aSchaff[i][instance+stoch11] = aSchaff[i-1][instance+stoch11];
                  aSchaff[i][instance+stoch12] = aSchaff[i-1][instance+stoch12]+0.5*(aSchaff[i][instance+stoch11]-aSchaff[i-1][instance+stoch12]);
               
         //
         //
         //
         //
         //

         minval = aSchaff[i][instance+stoch12];
         maxval = aSchaff[i][instance+stoch12];
            for (k=1; k<stcPeriod && (i-k)>=0; k++)
            {
               minval = MathMin(aSchaff[i-k][instance+stoch12],minval);
               maxval = MathMax(aSchaff[i-k][instance+stoch12],maxval);
            }
            maxval -= minval;
            if (maxval > 0)
                  aSchaff[i][instance+stoch21] = 100*((aSchaff[i][instance+stoch12]-minval)/maxval);
            else  aSchaff[i][instance+stoch21] = aSchaff[i-1][instance+stoch21];
                  aSchaff[i][instance+stoch22] = aSchaff[i-1][instance+stoch22]+0.5*(aSchaff[i][instance+stoch21]-aSchaff[i-1][instance+stoch22]);
   return(aSchaff[i][instance+stoch22]);
}

//
//
//
//
//

double workEma[][20];
double iEma(double price, double period, int r, int instanceNo=0)
{
   if (ArrayRange(workEma,0)!= Bars) ArrayResize(workEma,Bars); r = Bars-r-1;

   //
   //
   //
   //
   //
      
   double alpha = 2.0 / (1.0+period);
          workEma[r][instanceNo] = workEma[r-1][instanceNo]+alpha*(price-workEma[r-1][instanceNo]);
   return(workEma[r][instanceNo]);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
      if (first[i+2] == EMPTY_VALUE) {
          first[i]    = from[i];
          first[i+1]  = from[i+1];
          second[i]   = EMPTY_VALUE;
         }
      else {
          second[i]   = from[i];
          second[i+1] = from[i+1];
          first[i]    = EMPTY_VALUE;
         }
      }
   else
      {
         first[i]   = from[i];
         second[i]  = EMPTY_VALUE;
      }
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string stringUpperCase(string str) { string s = str; StringToUpper(s); return(s); }